% Compute the element load vectors corresponding to applied line load.
% Return an array of them so they may be assembled.
%    Call as
% evs = line_loads(feb, geom, u, ll)
%     geom=geometry field
%     u=displacement field
%     ll= body load
%
function evs = line_loads(self, geom, u, ll)
gcells = get(self.feblock,'gcells');
ngcells = length(gcells);
nfens = get(gcells(1),'nfens');
dim = get(geom,'dim');
if dim ~= 2
    error(['wrong dimension: need dim=1']);
end
% Pre-allocate the element matrices
evs(1:ngcells) = deal(elevec);
% Now loop over all gcells in the block
for i=1:ngcells
    edgeconn=get(gcells(i), 'edgeconn');
    if size(edgeconn,2)~=2
        error('Expected just two nodes per edge');
    end
    for j=1:size(edgeconn,1)
        x = gather(geom, edgeconn(j,:), 'values', 'noreshape'); % coordinates of nodes
        f=get_magn(ll,x);
        if ~isempty(f)
            h=norm(x(2,:)-x(1,:));
            evs(i) = set(evs(i), 'vec', h/2*reshape(f, dim*size(edgeconn,2), 1));
            evs(i) = set(evs(i), 'eqnums', gather(u, edgeconn(j,:), 'eqnums'));
        end
    end
end
return;

