% Compute the element load vectors corresponding to nonzero essential boundary
% conditions (settlement of the supports).
% Return an array of them so they may be assembled.
%    Call as
% evs = nz_ebc_loads(feb, geom, u)
%     geom=geometry field
%     u=displacement field
%
function evs = nz_ebc_loads(self, x, u)
gcells = get(self.feblock,'gcells');
ngcells = length(gcells);
nfens = get(gcells(1),'nfens');
dim = get(x,'dim');
% Pre-allocate the element matrices
evs(1:nfens*ngcells) = deal(elevec);
iev=1;
% Material
mat = get(self.feblock, 'mater');
matstates = get(self.feblock, 'matstates');
D = tangent_moduli (mat, struct ('ms',matstates{1,1}));
[N,W]=eig(D);
[ignore,ix]=sort(diag(W));
Wmean=(W(ix(1),ix(1))+W(ix(2),ix(2)))/2;
% Make the inverse map from finite element nodes to geometric cells
f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
gmap=get (f2gmap,'gcell_map');
fenodemap=get (f2gmap,'fenodes');
% Now loop over all finite element nodes
for nix=1:length(gmap)
    gl=gmap{nix};
    thisnn=fenodemap{nix};
    if ~isempty(gl) % This node has a element patch in this block
        % establish local numbering of all nodes of the patch @node nix
        lnmap =0*(1:length(gmap));
        for k=1:length(gl)
            kconn = get(gcells(gl(k)), 'conn'); % connectivity
            for j= 1:length(kconn)
                if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
                    lnmap(kconn(j))=kconn(j);
                end
            end
        end
        patchconn=lnmap(lnmap>0);
        % now store the local numbers
        for k=1:length(patchconn)
            lnmap(patchconn(k)) =k;
        end
        lnmap(thisnn) =length(patchconn)+1;
        c = gather(x, thisnn, 'values', 'noreshape'); % coordinates of central node
        % material tangent
        D = tangent_moduli (mat, struct ('ms',matstates{nix,1}));
        es=local_basis(self,[]); % calculate the local Cartesian basis at the node
        % preallocate the strain-displacement matrix
        Nspatialderavg=zeros((length(patchconn)+1),2);
        Apatch =0;
        tpatch =0;
        for k=1:length(gl)
            kconn = get(gcells(gl(k)), 'conn'); % connectivity
            lx = gather(x, kconn, 'values', 'noreshape'); % coordinates of nodes
            lx(:,1)=lx(:,1)-c(1);
            lx(:,2)=lx(:,2)-c(2);
            Nder = Ndermat_param (gcells(gl(k)), [1 1]/3);
            xl=lx*es(:,1:2);% local coordinates
            [Nspatialder,detF] = Ndermat_spatial (gcells(gl(k)), Nder, xl);
            Ae = detF * 1/2*1/3;% were taking one third of the triangle
            Apatch=Apatch+Ae;
            tpatch=tpatch+Ae*thickness(gcells(gl(k)), [1 1]/3);
            Nspatialderavg(lnmap(kconn),:)=Nspatialderavg(lnmap(kconn),:)+Ae*Nspatialder;
        end
        Bnodal= Blmat2 (gcells(gl(1)),Nspatialderavg/ Apatch,es);
        tpatch=tpatch/Apatch;
        Ke = Bnodal'*D*Bnodal * Apatch*tpatch;
        if (true)
            np=(length(patchconn)+1);
            Phi=zeros(dim*np, 6);
            lx = gather(x, [patchconn thisnn], 'values', 'noreshape'); % coordinates of nodes
            lx(:,1)=lx(:,1)-c(1);
            lx(:,2)=lx(:,2)-c(2);
            Phi(1:2:end-1, 1)=1;
            Phi(1:2:end-1, 2)=lx(:,1)';
            Phi(1:2:end-1, 3)=lx(:,2)';
            Phi(2:2:end, 4)=1;
            Phi(2:2:end, 5)=lx(:,1)';
            Phi(2:2:end, 6)=lx(:,2)';
            Kstab=self.stabfact*Wmean*(eye(size(Ke))-Phi*inv(Phi'*Phi)*Phi');
            %              [v,d]=eig(Ke); diag(d)
            %              [v,d]=eig(Kstab); diag(d)
            %             if norm (Kstab*uv)  > 1e-6
            %                 norm (Kstab*uv)
            %             end
            Ke = Ke + Kstab;
        end
        pu = gather(u, [patchconn thisnn], 'prescribed_values'); % prescribed values of displacement
        if norm (pu) >0
            evs(iev) = set(evs(iev), 'vec', -Ke*pu);
            evs(iev) = set(evs(iev), 'eqnums', gather(u, [patchconn thisnn], 'eqnums'));
            iev=iev+1;
        end
    end
end
return;
