% Compute the element load vectors corresponding to applied surface load.
% Return an array of them so they may be assembled.
%    Call as
% evs = surface_loads(feb, geom, u, ll)
%     geom=geometry field
%     u=displacement field
%     ll=surface load
%
function evs = surface_loads(self, geom, u, ll)
gcells = get(self.feblock,'gcells');
ngcells = length(gcells);
nfens = get(gcells(1),'nfens');
dim = get(geom,'dim');
if dim ~= 3
    error(['wrong dimension: need dim=3']);
end
% Pre-allocate the element matrices
evs(1:ngcells) = deal(elevec);
% Now loop over all gcells in the block
for i=1:ngcells
    faceconn=get(gcells(i), 'faceconn');
    nfn =size(faceconn,2);
    if (nfn ~= 3) 
        error('Expected three nodes per face');
    end
    for j=1:size(faceconn,1)
        x = gather(geom, faceconn(j,:), 'values', 'noreshape'); % coordinates of nodes
        f=get_magn(ll,x);
        if ~isempty(f)
            a=x(2,:)-x(1,:);
            b=x(3,:)-x(1,:);
            A=norm(skewmat(a)*b')/2;
            evs(i) = set(evs(i), 'vec', A/3*reshape(f, dim*size(faceconn,2), 1));
            evs(i) = set(evs(i), 'eqnums', gather(u, faceconn(j,:), 'eqnums'));
        end
    end
end
return;

