kappa_steel=[44 0; 0 44]; % N/K/s
kappa_tungsten=[163 0; 0 163]; % N/K/s
rho_steel = 7500*1e-9;% kg/mm^3
rho_tungsten = 19000*1e-9;% kg/mm^3
cv_steel =470*1e3*rho_steel;
cv_tungsten =134*1e3*rho_tungsten;
h=50*1e-3;
Ta=17;
Ts=-10;
Tt=84;
dt=2.5; % time step
tend= 100; % length of the time interval
t=0;
theta = 1.0; % generalized trapezoidal method
online_graphics= ~true;% plot the solution as it is computed?
num_integ_pts=1; % 1-point quadrature
[fens,gcells,groups,edge_gcells,edge_groups]=targe2_mesher({...
    'curve 1 line 0 0 50 0',...
    'curve 2 arc 50 0 80 0 center 65 -0.001 ',...
    'curve 3 line 80 0 110 0',...
    'curve 4 line 110 0 110 50',...
    'curve 5 line 110 50 65 50 ',...
    'curve 6 arc 65 50 65 70 center 65.001 60  ',...
    'curve 7 line 65 70 110 70',...
    'curve 8 line 110 70 110 85',...
    'curve 9 arc 110 85 65 120 center 110 120 ',...
    'curve 10 line 65 120 0 120',...
    'curve 11 line 0 120 0 85',...
    'curve 12 arc 0 85 0 35 center -0.001 60 rev',...
    'curve 13 line 0 35 0 0',...
    'curve 14 line 110, 50, 160, 50',...
    'curve 15 line 160, 50, 160, 70',...
    'curve 16 line 160, 70, 110, 70',...
    ['subregion 1  property 1 boundary '...
    ' 1 2 3 4 5 6 7 8 9 10 11 12 13'],...
    ['subregion 2  property 2 boundary '...
    ' -5 -6 -7 14 15 16'],...
    ['m-ctl-point constant 3']
    }, 1.0);
prop_steel=...
    property_diffusion (struct('conductivity',kappa_steel,...
    'specific_heat',cv_steel,'source',0.0));
mater_steel=mater_diffusion (struct('property',prop_steel));
feb_steel = feblock_diffusion (struct ('mater',mater_steel,...
    'gcells',gcells(groups{1}),...
    'integration_rule',tri_rule(num_integ_pts)));
prop_tungsten=...
    property_diffusion (struct('conductivity',kappa_tungsten,...
    'specific_heat',cv_tungsten,'source',0.0));
mater_tungsten=mater_diffusion (struct('property',prop_tungsten));
feb_tungsten = feblock_diffusion (struct ('mater',mater_tungsten,...
    'gcells',gcells(groups{2}),...
    'integration_rule',tri_rule(num_integ_pts)));
edge_gcells=edge_gcells([edge_groups{[(1:4) (8:16)]}]);
efeb = feblock_diffusion (struct ('mater',mater_steel,...
    'gcells',edge_gcells,...
    'integration_rule',gauss_rule(1,num_integ_pts),...
    'surface_transfer', h));
geom = field(struct('name',['geom'], 'dim', 2, 'fens',fens));
tempn=field(struct('name',['tempn'], 'dim', 1, 'nfens',...
    get(geom,'nfens')));
amb = clone(tempn, ['amb']);
for i= 1:length(edge_gcells)
    conn = get(edge_gcells(i),'conn');
    amb = set_ebc(amb, conn, conn*0+1, [], conn*0+Ta);
end
amb = apply_ebc (amb);

tempn = numbereqns (tempn);
tempn = scatter_sysvec(tempn,gather_sysvec(tempn)*0+Ts);
gc = get(feb_tungsten,'gcells');
for i= 1:length(gc)
    conn = get(gc(i),'conn');
    tempn = scatter(tempn, conn, conn'*0+Tt);
end
K = start (sparse_sysmat, get(tempn, 'neqns'));
K = assemble (K, conductivity(feb_steel, geom, tempn));
K = assemble (K, conductivity(feb_tungsten, geom, tempn));
K = assemble (K, surface_transfer(efeb, geom, tempn));
Km = get(K,'mat');
C = start (sparse_sysmat, get(tempn, 'neqns'));
C = assemble (C, capacity(feb_steel, geom, tempn));
C = assemble (C, capacity(feb_tungsten, geom, tempn));
Cm = get(C,'mat');

if online_graphics
    gv=graphic_viewer;
    dcm=data_colormap(struct('range',[Ts,Tt],'colormap',jet));
end
minmaxT = []
while t<tend+0.1*dt % Time stepping
    if online_graphics
        gv=clear (gv,[]);
        gv=reset (gv,[]);
        set(gca,'FontSize', 14)
        T=get(tempn,'values');
        colorfield=field(struct('name',['colorfield'],'data',...
            map_data(dcm, T)));
        geomT=field(struct ('name', ['geomT'], ...
            'data',[get(geom,'values'), get(tempn,'values')]));
        for i=1:length (gcells)
            draw(gcells(i),gv,struct('x',geomT, 'u',0*geomT,...
                'colorfield',colorfield, 'shrink',1.0));
            draw(gcells(i),gv,struct('x',geom, 'u',0*geom, ...
                'facecolor','none'));
        end
        camset (gv,[-671.7042 -895.3086 695.7305 58.8095 ...
            56.7155 2.9101 0 0 1 7.3342])
        xlabel('X [mm]');        ylabel('Y [mm]');
        zlabel('Temperature [degrees C]')
        figure(gcf);
        title (['Time =' num2str(t)]); hold off; pause(0.1);
        %         saveas(gcf, ['shrinkfit-' num2str(t) '.png'], 'png');
    end
    tempn1 = tempn;
    F = start (sysvec, get(tempn, 'neqns'));
    F=assemble(F, surface_transfer_loads(efeb,geom,tempn,amb));
    Tn=gather_sysvec(tempn);
    minmaxT= [minmaxT; min(Tn),max(Tn)]
    Tn1 = (1/dt*Cm+theta*Km) \ ((1/dt*Cm-(1-theta)*Km)*Tn+...
        get(F,'vec'));
    tempn = scatter_sysvec(tempn1,Tn1);
    t=t+dt;
end
figure
plot((1:size(minmaxT, 1))*dt,minmaxT,'linewidth', 3)
set(gca,'FontSize', 14); grid on
xlabel('Time in seconds')
ylabel('Temperature in degrees Celsius')