% Merge two meshes together by gluing together nodes within tolerance.
function [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance)
    xyz = get(fens1(1),'xyz');
    xyz1 = zeros(length(fens1),length(xyz));
    for i=1:length(fens1)
        xyz1(i,:) =get(fens1(i),'xyz');
    end
    xyz = get(fens2(1),'xyz');
    xyz2 = zeros(length(fens2),length(xyz));
    for i=1:length(fens2)
        xyz2(i,:) =get(fens2(i),'xyz');
    end
    % Mark nodes from the first array that are duplicated in the second
    for i=1:length(fens1)
        xyz =xyz1(i,:);
        for k=1:length(fens2)
            if norm(xyz-xyz2(k,:)) <tolerance
                fens1(i)=fenode(struct('id',-k,'xyz',xyz)); 
            end
        end
    end
    % Generate IDs of the nodes
    fens(1:length(fens1)+length(fens2)) = deal(fenode);
    mid=1;
    for i=1:length(fens1)
        id = get(fens1(i),'id');
        if id>0
            fens1(i)=fenode(struct('id',mid,'xyz',get(fens1(i),'xyz')));
            fens(mid)=fenode(struct('id',mid,'xyz',get(fens1(i),'xyz')));
            mid=mid+1;
        end
    end
    for k=1:length(fens2)
        id = get(fens2(k),'id');
        if id>0
            fens2(k)=fenode(struct('id',mid,'xyz',get(fens2(k),'xyz')));
            fens(mid)=fenode(struct('id',mid,'xyz',get(fens2(k),'xyz')));
            mid=mid+1;
        end
    end
    fens =fens(1:mid-1);
    % Renumber the cells
    for i=1:length(gcells1)
        conn=get(gcells1(i),'conn');
        for j=1:length(conn)
            id=get(fens1(conn(j)),'id');
            if id <=0
                id=get(fens2(abs(id)),'id');
            end
            conn(j) =id;
        end
        gcells1(i)=set(gcells1(i),'conn',conn);
    end
    for i=1:length(gcells2)
        conn=get(gcells2(i),'conn');
        for j=1:length(conn)
            conn(j) =get(fens2(conn(j)),'id');
        end
        gcells2(i)=set(gcells2(i),'conn',conn);
    end
end