function set_figure_state(h,state)
    % set_figure_state sets a figures state to maximimze,minimize or restore
    % set_figure_state(H,STATE): H is figure handle or a vector of figure handles
    %                          STATE is a string or cell array of strings-
    %                               'maximize' - maximize figure
    %                               'minimize' - minimize figure
    %                               'restore'- restore figure
    %                           if STATE is a string the state is applied to
    %                           all H. If state is a cell array the length STATE
    %                           must equal that of H and each state is applied
    %                           individually.
    %  Examples:
    %   h= figure;
    %   s = 'maximize';
    %   set_figure_state(h,s) %sets h to maximize
    %
    %   h(1) = figure;
    %   h(2) = figure;
    %   s = 'minimize';
    %   set_figure_state(h,s) %minimizes both figures
    %
    %   h(1) = figure;
    %   h(2) = figure;
    %   s = {'minimize','maximize'};
    %   set_figure_state(h,s) %minimizes h(1) and maximizes h(2)
    %   set_figure_state(h,'restore') %restores both windowa
    % Notes: Figures must have 'Visible' set to 'on' and not be docked for
    % set_figure_state to work.
    %
    % set_figure_state is just a renamed function
    % setWindowState (c)  	Matthew Whitaker
    %Revisions: 01/09/06- Call the methods on the event thread using awtinvoke

    drawnow; %need to make sure that the figures have been rendered or Java error can occur

    %check input argument number
    error(nargchk(2, 2, nargin, 'struct'));

    %is JVM available
    if ~usejava('jvm')
        error('setWindowState requires Java to run.');
    end

    [j,s] = parseInput;

    for i = 1:length(j)
        awtinvoke(j{i},[s{i},'()']);
    end %for

    function [j,s] = parseInput
        % is h all figure handles
        if ~all(ishandle(h)) || ~isequal(length(h),length(findobj(h,'flat','Type','figure')))
            error('All input handles must be valid figure handles');
        end %if

        %handle state argument
        if ischar(state)
            %make it a cell
            s = cellstr(repmat(state,[length(h),1]));

        elseif iscellstr(state)
            if length(state) ~= length(h)
                error('Cell array of strings: state must be same length as figure handle input');
            end %if
            s = state;
        else
            error('state must be a character array or a cell array of strings');
        end %if

        %check that the states are all valid
        if ~all(ismember(s,{'maximize','minimize','restore'}))
            error('Invalid states entered')
        end %if

        if length(h) == 1
            j{1} = get(h,'javaframe');
        else
            j = get(h,'javaframe');
        end %if

    end %parseInput

end %setWindowState